// This file is part of StarClassifier. Please see StarClassifier.java or 
// http://starclassifier.udd.be/ for license information. 
// Copyright (C) 2008 Niklas Udd

// DatabaseConfiguration is a class that handles configuration of a database as well as
// interaction with that database.

package be.udd.starclassifier;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.Statement;
import java.sql.SQLException;

public class DatabaseConfiguration {
	private String driverClass;
	private String url;
	private String loadSql;
	private String saveSql;
	private String id;
	
	private static Connection connection = null;
	private static Statement statement = null; 	

	public DatabaseConfiguration(String driverClass, String url, String loadSql, String saveSql, String id) {
		this.driverClass = driverClass;
		this.url = url;
		this.loadSql = loadSql;
		this.saveSql = saveSql;
		this.id = id;
	}	

	public void loadDriver() throws Exception {
		Class.forName(driverClass).newInstance();
	}

	public void connect() throws SQLException {
		connection = DriverManager.getConnection(url);
	}
	
	public ResultSet loadData() throws SQLException {
		statement = connection.createStatement();		
		return statement.executeQuery(loadSql);
	}
	
	public boolean saveData(Object[] objectArr) throws SQLException {
		statement = connection.prepareStatement(saveSql);
		for(int i = 0; i < objectArr.length; i++) {
			((PreparedStatement)statement).setObject(i + 1, objectArr[i]);
		}
		return ((PreparedStatement)statement).execute();
	}
	
	public void disconnect() {
		try {
			if(statement != null)
				statement.close();
			connection.close();
		} catch(SQLException e) {}
	}
	
	public String getId() {
		return id;
	}
}