// This file is part of StarClassifier. Please see StarClassifier.java or 
// http://starclassifier.udd.be/ for license information. 
// Copyright (C) 2008 Niklas Udd

// This files reads a configuration file and translates it into an object.

package be.udd.starclassifier;

import java.io.BufferedReader;
import java.io.FileReader;

public class DatabaseConfigurationLoader {
	public static DatabaseConfiguration load(String filename) throws Exception {
		String driverClass = null;
		String url = null;
		String loadSql = null;
		String saveSql = null;
		String id = null;
		
		FileReader fileReader = new FileReader(filename);
		BufferedReader bufferedReader = new BufferedReader(fileReader);
		
		try {
			String line;
			while((line = bufferedReader.readLine()) != null) {
				if(line.length() > 0 && line.charAt(0) == '@') {
					String command = line.substring(0,line.indexOf(' '));
					String data = line.substring(line.indexOf(' ') + 1);
					
					if(command.equals("@driver")) {
						driverClass = data;
					} else if(command.equals("@url")) {
						url = data;
					} else if(command.equals("@load_sql")) {
						loadSql = data;
					} else if(command.equals("@save_sql")) {
						saveSql = data;
					} else if(command.equals("@id")) {
						id = data;
					}
				}
			}
		} catch(Exception e) {
			try {
				bufferedReader.close();
				fileReader.close();
			} catch(Exception e2) {}
			throw new Exception("Unable to read database configuration from file! (Is the file valid?)");
		}
		bufferedReader.close();
		fileReader.close();
		
		DatabaseConfiguration databaseConfiguration = new DatabaseConfiguration(driverClass, url, loadSql, saveSql, id);
		return databaseConfiguration;
	}
}