// This file is part of StarClassifier. Please see StarClassifier.java or 
// http://starclassifier.udd.be/ for license information. 
// Copyright (C) 2008 Niklas Udd

// This class represents a leaf in a decision tree. Since a leaf is a valid tree in 
// itself this class extends DecisionTree.

package be.udd.starclassifier;

public class DecisionLeaf extends DecisionTree {
	private boolean isDefaultClass;
	private double total;
	private double error;
	
	public DecisionLeaf(String className, boolean isDefaultClass, double total, double error) {
		super(className);
		this.isDefaultClass = isDefaultClass;
		this.total = total;
		this.error = error;
	}
	
	public boolean isDefaultClass() {
		return isDefaultClass;
	}
	
	public double getNumberOfExamples() {
		return total;
	}
	
	public double getNumberOfIncorrectExamples() {
		return error;
	}

	public double getNumberOfExamplesInDefaultClass() {
		if(isDefaultClass)
			return total - error;
		else
			return error;
	}
	
	public int size() {
		return 1;
	}
	
	public String toString(String prefix) {
		return prefix + getName() + " (" + getNumberOfExamples() + ((getNumberOfIncorrectExamples() > 0)?("/" + getNumberOfIncorrectExamples()):"") + ")\n";
	}
	
	public String toWekaString(String prefix) {
		return getName() + " (" + getNumberOfExamples() + ((getNumberOfIncorrectExamples() > 0)?("/" + getNumberOfIncorrectExamples()):"") + ")\n";
	}
}