// This file is part of StarClassifier. Please see StarClassifier.java or 
// http://starclassifier.udd.be/ for license information. 
// Copyright (C) 2008 Niklas Udd

// This class represents a node in a decision tree. It implements functionality that
// is equal between different node types.

package be.udd.starclassifier;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

public abstract class DecisionNode extends DecisionTree {
	private Vector<String> edgesInOrder;
	private Hashtable<String,DecisionTree> children;
	
	public DecisionNode(String name) {
		super(name);
		edgesInOrder = new Vector<String>();
		children = new Hashtable<String,DecisionTree>();
	}
		
	public String getCondition() {
		return getName();
	}
	
	public String getCondition(String edge) {
		return getName() + " = " + edge;
	}
	
	public void addChild(String edge, DecisionTree child) {
		edgesInOrder.add(edge);
		children.put(edge, child);
	}

	public Enumeration<String> getChildren() {
		return children.keys();
	}
	
	public DecisionTree classify(String edge) {
		DecisionTree restOfTree = children.get(edge);
		return restOfTree;
	}

	public int size() {
		Enumeration<String> allKeys = children.keys();

		int result = 0;
		while(allKeys.hasMoreElements()) {
			String key = allKeys.nextElement();
			result += children.get(key).size();
		}
		return result;
	}	

	public double getNumberOfExamples() {
		Enumeration<String> allKeys = children.keys();

		double result = 0;
		while(allKeys.hasMoreElements()) {
			String key = allKeys.nextElement();
			result += children.get(key).getNumberOfExamples();
		}
		return result;	
	}
	
	public double getNumberOfIncorrectExamples() {
		Enumeration<String> allKeys = children.keys();

		double result = 0;
		while(allKeys.hasMoreElements()) {
			String key = allKeys.nextElement();
			result += children.get(key).getNumberOfIncorrectExamples();
		}
		return result;	
	}
	
	public double getNumberOfExamplesInDefaultClass() {
		Enumeration<String> allKeys = children.keys();

		double result = 0;
		while(allKeys.hasMoreElements()) {
			String key = allKeys.nextElement();
			result += children.get(key).getNumberOfExamplesInDefaultClass();
		}
		return result;	
	}
	
	public String toString(String prefix) {
		String result = "";
		String key;
		for(int i = 0; i < edgesInOrder.size(); i++) {
			key = edgesInOrder.get(i);		
			result += prefix + this.getCondition(key);
			result += "\n";
			result += children.get(key).toString(prefix + "   ");
		}
		return result;
	}
	
	public String toWekaString(String prefix) {
		String result = "";
		String key;
		for(int i = 0; i < edgesInOrder.size(); i++) {
			key = edgesInOrder.get(i);
			result += prefix + this.getCondition(key);
			if(children.get(key) instanceof DecisionLeaf) {
				DecisionLeaf leaf = (DecisionLeaf)children.get(key);
				result += ": " + leaf.toWekaString();
			} else {
				result += "\n";
				result += children.get(key).toWekaString(prefix + "|   ");
			}
		}
		return result;		
	}
}