// This file is part of StarClassifier. Please see StarClassifier.java or 
// http://starclassifier.udd.be/ for license information. 
// Copyright (C) 2008 Niklas Udd

// This class classifies an example with the help of a decision tree. 

package be.udd.starclassifier;

import java.util.Enumeration;

public class DecisionTreeClassifier {
	public static Classification classify(DecisionTree decisionTree, Example example) {
		// Set null as the parent decision tree.
		return classify(decisionTree, example, null);
	}
	
	public static Classification classify(DecisionTree decisionTree, Example example, DecisionTree parentDecisionTree) {
		Classification classification = null;
		if(decisionTree instanceof DecisionLeaf) {
			// A leaf has been found in the tree
			double numberOfExamples = decisionTree.getNumberOfExamples();
			double numberOfExamplesInDefaultClass = decisionTree.getNumberOfExamplesInDefaultClass();

			if(numberOfExamples == 0) {
				// The actual leaf had no examples, get distribution from parent node			
				if(parentDecisionTree == null)
					throw new RuntimeException("This leaf has no examples and no parent node. What kind of decision tree is this?");

				numberOfExamples = parentDecisionTree.getNumberOfExamples();
				numberOfExamplesInDefaultClass = parentDecisionTree.getNumberOfExamplesInDefaultClass();
			}
			
			classification = new Classification(numberOfExamples, numberOfExamplesInDefaultClass);
		} else {
			// A leaf has not been found -- traverse
			String attribute = decisionTree.getName();
			String value = example.getValue(attribute);
			
			if(value != null) {
				// Non-null value
				DecisionTree newDecisionTree;
				if(decisionTree instanceof DecisionNodeNumeric)
					newDecisionTree = ((DecisionNodeNumeric)decisionTree).classifyByValue(value);
				else
					newDecisionTree = ((DecisionNode)decisionTree).classify(value);
							
				// The decision tree will be null if the value was unknown
				if(newDecisionTree != null) 
					classification = classify(newDecisionTree, example, decisionTree);
			}
			
			if(classification == null) {
				// Null value or unknown value
				Enumeration<String> allChildren = ((DecisionNode)decisionTree).getChildren();
				double totalDefaultClassProbability = 0;
				while(allChildren.hasMoreElements()) {
					// Check all bransches of the tree
					String child = allChildren.nextElement();
					DecisionTree thisDecisionTree = ((DecisionNode)decisionTree).classify(child);
					Classification thisClassification = classify(thisDecisionTree, example, decisionTree);

					// Update the global distribution with the result from this bransch
					double thisNumberOfExamplesInBransch = thisDecisionTree.getNumberOfExamples();
					double thisDefaultClassProbability = thisClassification.getDefaultClassProbability();
					totalDefaultClassProbability += (thisNumberOfExamplesInBransch * thisDefaultClassProbability);
				}
				
				classification = new Classification(decisionTree.getNumberOfExamples(), totalDefaultClassProbability);
			}
		}
		return classification;
	}
}