// This file is part of StarClassifier. Please see StarClassifier.java or 
// http://starclassifier.udd.be/ for license information. 
// Copyright (C) 2008 Niklas Udd

// DecisionTreeLoader needs this file when it loads a decision tree. It is an
// intermediate representation of a decision tree. 

package be.udd.starclassifier;

public class NodeLineRepresentation {
	public final static int NUMERIC_NODE = 1;
	public final static int CATEGORIC_NODE = 2;
	
	private boolean hasLeaf;
	private int nodeType;
	
	private int level;
	private String attribute;
	private String value;
	private String operator;
	private int splitValue;
	private String className;
	private double total;
	private double error;
	
	public NodeLineRepresentation(int level, String attribute, String value) {
		hasLeaf = false;
		nodeType = CATEGORIC_NODE;
		
		this.level = level;
		this.attribute = attribute;
		this.value = value;
	}
	
	public NodeLineRepresentation(int level, String attribute, String operator, int splitValue) {
		hasLeaf = false;
		nodeType = NUMERIC_NODE;
		
		this.level = level;
		this.attribute = attribute;
		this.operator = operator;
		this.splitValue = splitValue;		
	}
		
	public void setLeaf(String className, double total, double error) {
		hasLeaf = true;
		
		this.className = className;
		this.total = total;
		this.error = error;
	}
	
	public int getType() {
		return nodeType;
	}
	
	public String getAttribute() {
		return attribute;
	}
	
	public int getSplitValue() {
		return splitValue;
	}
	
	public int getLevel() {
		return level;
	}
	
	public boolean hasLeaf() {
		return hasLeaf;
	}
	
	public String getClassName() {
		return className;
	}
	
	public double getTotal() {
		return total;
	}
	
	public double getError() {
		return error;
	}
	
	public String getValue() {
		return value;
	}
	
	public String getOperator() {
		return operator;
	}
	
	public String toString() {
		String result = attribute;
		if(nodeType == NUMERIC_NODE)
			result += " " + operator + " " + splitValue;
		else
			result += " = " + value;
		if(hasLeaf)
			result += ": " + className + "(" + total + "/" + error + ")";
		return result;
	}
}