/*
    StarClassifier 
    Version: 0.1
    Release date: 2008-02-10
    E-mail: starclassifier@udd.be
    Web site: http://starclassifier.udd.be/
    Copyright (C) 2008 Niklas Udd

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package be.udd.starclassifier;

import java.util.Vector;

public class StarClassifier {
	public static void main(String[] args) {
		// Check input arguments
		if(args.length < 3) {
			System.err.println("Incorrect number of arguments!");
			System.err.println("Usage: java be.udd.starclassifier.StarClassifier TREE CLASS CONFIG1 [CONFIG2 [CONFIG3 [...]]]");
			System.exit(1);
		}
		
		String treeFile = args[0];
		String defaultClassValue = args[1]; 
		int numExamplesClassified = 0;
		try {		
			// Load the decision tree
			System.out.println("Loading tree from file " + treeFile);
			DecisionTree tree = DecisionTreeLoader.load(treeFile, defaultClassValue);
			
			// Loop through the config files and classify all examples
			for(int i = 2; i < args.length; i++) {
				// Read the config file and load all examples into memory 
				System.out.println("Classifying examples described by file " + args[i]);
				DatabaseConfiguration dbConf = DatabaseConfigurationLoader.load(args[i]);
				dbConf.loadDriver();
				dbConf.connect();
				Vector<Example> exampleSet = ExampleSetLoader.loadFromDb(dbConf);
	
				// Go through all examples and do the classification
				while(exampleSet.size() > 0) {
					Example example = exampleSet.remove(0);
					Classification thisClassification = DecisionTreeClassifier.classify(tree, example);
					ClassificationSaver.saveClassification(dbConf, example, thisClassification);
					numExamplesClassified++;
				}
				dbConf.disconnect();
			}			
		} catch(Exception e) {
			System.err.println("Execution was stopped due to an error:\n" + e);
			System.exit(2);
		}

		// Output a special exit code if no exampels were classified
		if(numExamplesClassified == 0) {
			System.exit(100);
		}
	}
}